/*
This function loads a bitmap for a desktop icon
Arguments: pointer to the pointer where to allocate memory,
string that contains a path to the icon file
*/

unsigned load_icon_bitmap(unsigned char **icon, char *icon_file)
{
  FILE *fp;
  *icon = malloc(100*sizeof(char));

  if(!icon) return 0;
  
  if(fp=fopen(icon_file, "rb"))
  {
    fread(*icon, sizeof(char), 100, fp);
    fclose(fp);
  }
  else
  {
    free(icon);
    return 0;
  }
  return 1;
}

/*
Loads icon
arguments: pointer to the icon struct, path to the icon bitmap file,
a string that contains the text of icon
*/
void load_icon(struct icon *icon, char *icon_file, unsigned char *text)
{
  unsigned char *tp;  
  load_icon_bitmap(&icon->bitmap, icon_file);
  if(text)
  {  
    icon->text = malloc((1+strlen(text))*sizeof(char));
    tp = icon->text;
    while(*tp++ = *text++);
  }
}
 
void init_desktop(void)
{
  char *static_icons[][2] =
  {
    {
      "icons/runprmpt.ico", "Run program"
    },
    {
      "icons/fileman.ico", "File manager"
    },
    {
      "icons/cpanel.ico", "Control panel"
    },
    {
      "icons/clock.ico", "Clock"
    }
  };
  unsigned x = 20;
  windows = malloc(sizeof(struct prog_window*));
  window_visibility_order = malloc(sizeof(int));
  desktop_icons = malloc((sizeof(static_icons)/sizeof(char*))*sizeof(struct icon));
  default_icons = malloc(sizeof(char**));

  default_icons[0] = malloc(sizeof(char*));
  load_icon_bitmap(&default_icons[0], "icons/wndwdflt.ico");

  // load static icons
  for(;desktop_icon_count<(sizeof(static_icons)>>1)/sizeof(char*);desktop_icon_count++,x+=60)
  {
    load_icon(&desktop_icons[desktop_icon_count], static_icons[desktop_icon_count][0], static_icons[desktop_icon_count][1]);
    desktop_icons[desktop_icon_count].x=x;
    desktop_icons[desktop_icon_count].y=20;    
  }
/*  desktop_icon_count++;
  load_icon(&desktop_icons[desktop_icon_count-1], "icons/runprmpt.ico", "Run program");
  desktop_icons[desktop_icon_count-1].x=20;
  desktop_icons[desktop_icon_count-1].y=20;
  desktop_icons[desktop_icon_count-1].value = 0;

  desktop_icon_count++;
  load_icon(&desktop_icons[desktop_icon_count-1], "icons/fileman.ico", "File manager");
  desktop_icons[desktop_icon_count-1].x=80;
  desktop_icons[desktop_icon_count-1].y=20;
  desktop_icons[desktop_icon_count-1].value = 1;

  desktop_icon_count++;
  load_icon(&desktop_icons[desktop_icon_count-1], "icons/cpanel.ico", "Control panel");
  desktop_icons[desktop_icon_count-1].x=140;
  desktop_icons[desktop_icon_count-1].y=20;
  desktop_icons[desktop_icon_count-1].value = 1;  */
  
}

void draw_icon(unsigned int x, unsigned int y, unsigned char *icon, unsigned char *text, unsigned char color)
{
  unsigned int n=20;
  unsigned int m;
  unsigned int xc;

  while(n--)
  {
    for(m=5,xc=x;m--;)
    {
      if(*icon & 0x80)
        put_pixel(xc, y, color);
      xc++;
      if(*icon & 0x40)
        put_pixel(xc, y, color);
      xc++;      
      if(*icon & 0x20)
        put_pixel(xc, y, color);
      xc++;
      if(*icon & 0x10)
        put_pixel(xc, y, color);
      xc++;
      if(*icon & 0x08)
        put_pixel(xc, y, color);
      xc++;
      if(*icon & 0x04)
        put_pixel(xc, y, color);
      xc++;
      if(*icon & 0x02)
        put_pixel(xc, y, color);
      xc++;
      if(*icon & 0x01)
        put_pixel(xc, y, color);
      xc++;
      icon++;
    }
    y++;
  }

  gprint_text(x, y+16-_FONT_HEIGHT, text, color, _FONT_HEIGHT, x+_WINDOW_ICONS_MARGIN, 0);
}

void save_desktop_shortcut(unsigned shortcut)
{
  char coords[5];
  unsigned n;
  FILE *fp;
  char shortcut_file_path[81];

  sprintf(shortcut_file_path, "DESKTOP\\%s", desktop_shortcuts[shortcut].shortcut_file);  
  
  fp = fopen(shortcut_file_path, "w");
  
  for(n=0;desktop_shortcuts[shortcut].path[n];n++)
    fputc(desktop_shortcuts[shortcut].path[n], fp);
  fputc(';', fp);
  
  for(n=0;desktop_shortcuts[shortcut].text[n];n++)
    fputc(desktop_shortcuts[shortcut].text[n], fp);
  fputc(';', fp);

  sprintf(coords, "%i", desktop_shortcuts[shortcut].x);
  for(n=0;coords[n];n++)
    fputc(coords[n], fp);
  fputc(';', fp);
  
  sprintf(coords, "%i", desktop_shortcuts[shortcut].y);
  for(n=0;coords[n];n++)
    fputc(coords[n], fp);
  fputc(';', fp);
  
  sprintf(coords, "%i", desktop_shortcuts[shortcut].ask_args);
  for(n=0;coords[n];n++)
    fputc(coords[n], fp);
  fputc(';', fp);
  
  if(desktop_shortcuts[shortcut].icon_path)
  {
    for(n=0;desktop_shortcuts[shortcut].icon_path[n];n++)
      fputc(desktop_shortcuts[shortcut].icon_path[n], fp);
  }

  fclose(fp);
}

unsigned load_desktop_shortcuts(void)
{
  FILE *fp;
  FILE *icon;
  unsigned int m;
  int c;
  char coords[5];
  char icon_path[81];
  struct find_t fileinfo;
  unsigned rc;
  if(!(desktop_shortcuts = malloc(sizeof(struct desktop_shortcut))))
    return 0;

  rc = _dos_findfirst("DESKTOP\\*.lnk", _A_NORMAL, &fileinfo); 
  while(!rc)
  {
    if(shortcut_count)
    {
      if(!(desktop_shortcuts = realloc(desktop_shortcuts, (1+shortcut_count)*sizeof(struct desktop_shortcut))))
        return 0;
    }
      
    sprintf(icon_path, "DESKTOP\\%s", fileinfo.name);
    fp = fopen(icon_path, "r");

    sprintf(desktop_shortcuts[shortcut_count].shortcut_file, "%s", fileinfo.name);

    for(m=0;(c = fgetc(fp)) != ';';m++)
      desktop_shortcuts[shortcut_count].path[m] = c;
    desktop_shortcuts[shortcut_count].path[m]=0;

    for(m=0;(c = fgetc(fp)) != ';';m++)
      desktop_shortcuts[shortcut_count].text[m] = c;
    desktop_shortcuts[shortcut_count].text[m]=0;      

    for(m=0;(c = fgetc(fp)) != ';';m++)
      coords[m] = c;
    coords[m] = 0;
    desktop_shortcuts[shortcut_count].x = atoi(coords);

    for(m=0;(c = fgetc(fp)) != ';';m++)
      coords[m] = c;
    coords[m] = 0;      
    desktop_shortcuts[shortcut_count].y = atoi(coords);

    for(m=0;(c = fgetc(fp)) != ';';m++)
      coords[m] = c;
    coords[m] = 0;      
    desktop_shortcuts[shortcut_count].ask_args = atoi(coords);    

    for(m=0;(c = fgetc(fp)) != -1;m++)
      icon_path[m] = c;
    icon_path[m] = 0;

    fclose(fp);

    if(icon = fopen(icon_path, "rb"))
    {
      if(!(desktop_shortcuts[shortcut_count].icon = malloc(100*sizeof(char))))
        return 0;
      fread(desktop_shortcuts[shortcut_count].icon, sizeof(char), 100, icon);
      fclose(icon);
      
      if(!(desktop_shortcuts[shortcut_count].icon_path = malloc(81*sizeof(char))))
        return 0;
      sprintf(desktop_shortcuts[shortcut_count].icon_path, "%s", icon_path);
    }
    else
    {
      desktop_shortcuts[shortcut_count].icon = 0;
      desktop_shortcuts[shortcut_count].icon_path = 0;
    }

    shortcut_count++;
    rc = _dos_findnext(&fileinfo);     
  }
  return 1;
}

void draw_desktop_shortcuts(void)
{
  unsigned int n = shortcut_count;
  unsigned char color;
  while(n--)
  {
    if(icon_selected == desktop_icon_count + n)
    {
      color = _ICON_ACOLOR;
      draw_borders(desktop_shortcuts[n].x, desktop_shortcuts[n].y, 40, 35, 2);            
    }
    else
    {
      color = _ICON_ICOLOR;
      draw_borders(desktop_shortcuts[n].x, desktop_shortcuts[n].y, 40, 35, 4);                  
    }
      
    if(desktop_shortcuts[n].icon)
      draw_icon(desktop_shortcuts[n].x, desktop_shortcuts[n].y, desktop_shortcuts[n].icon, desktop_shortcuts[n].text, color);
    else
      draw_icon(desktop_shortcuts[n].x, desktop_shortcuts[n].y, default_icons[0], desktop_shortcuts[n].text, color);    
  }
}


void draw_desktop_icons(void)
{
  unsigned int n = desktop_icon_count;
//  unsigned int x=20;
//  unsigned int y=20;
  while(n--)
  {
    if(icon_selected != n)
    {
      draw_borders(desktop_icons[n].x, desktop_icons[n].y, 40, 35, 4);      
      draw_icon(desktop_icons[n].x, desktop_icons[n].y, desktop_icons[n].bitmap, desktop_icons[n].text, _ICON_ICOLOR);
    }
    else
    {
      draw_borders(desktop_icons[n].x, desktop_icons[n].y, 40, 35, 2);
      draw_icon(desktop_icons[n].x, desktop_icons[n].y, desktop_icons[n].bitmap, desktop_icons[n].text, _ICON_ACOLOR);
    }
  }
}

void draw_window_icons(void)
{
  unsigned int n;
  unsigned int x = _WINDOW_ICONS_LEFT;
  for(n=0;n<window_count;n++)
  {
    if(windows[n]->minimized)
    {
      draw_borders(x, _RES_Y-40, _WINDOW_ICONS_MARGIN-2, 39, 0);
      if(windows[n]->icon)
        draw_icon(x, _RES_Y-40, windows[n]->icon, windows[n]->title, _WINDOW_ICON_COLOR);
      else
        draw_icon(x, _RES_Y-40, default_icons[0], windows[n]->title, _WINDOW_ICON_COLOR);
      x+=_WINDOW_ICONS_MARGIN;
    }
  }  
}

void draw_desktop(void)
{
  unsigned int x=0;
  unsigned int y=0;

//  for(;x<_RES_X;x++,y=0)
//    for(;y<_RES_Y;y++)
  if(_VIDEO_MODE == 0x10 || _VIDEO_MODE == 0x12)
    fill_screen_ega4(_DESKTOP_BGCOLOR);
  else if(_VIDEO_MODE == 0x0F || _VIDEO_MODE == 0x11)
    fill_screen_ega2(_DESKTOP_BGCOLOR);
  else if(_VIDEO_MODE == 0x06)
    fill_screen_cga(_DESKTOP_BGCOLOR);
  else
    for(;y<_RES_Y;y++,x=0)
      for(;x<_RES_X;x++)
        put_pixel(x, y, _DESKTOP_BGCOLOR);

//  gprint_text(0, 0, "FreeDOSShell v. 0.3 preview-alpha - build 85", 0x00, 8, _RES_X, 0);

  draw_desktop_icons();
  draw_desktop_shortcuts();
}

void draw_windows(void)
{
  unsigned int n;
  draw_window_icons();  
  for(n=0;n<window_count;n++)
  {
    if(active_window == window_visibility_order[n])
    {
      if(!windows[window_visibility_order[n]]->minimized && !windows[window_visibility_order[n]]->hidden)        
        draw_window(windows[window_visibility_order[n]], 1, 0);
    }
    else
      if(!windows[window_visibility_order[n]]->minimized)    
        draw_window(windows[window_visibility_order[n]], 0, 0);
  }
}

void draw_screen(void)
{
  draw_desktop();
  draw_windows();
}

void update_titlebar(struct prog_window *window)
{
  if(titlebar_drag)
  {
    titlebar_drag->x0 = window->x;
    titlebar_drag->y0 = window->y;
    titlebar_drag->x1 = window->x+window->width;
    titlebar_drag->y1 = window->y+_FONT_HEIGHT+2;
  }
}

void set_active_window(int window, unsigned draw)
{
  unsigned int n;
  if(window>=0)
  {
/*    if(window_count>1)
    {
      window_visibility_order[window_count-2] = active_window;
      window_visibility_order[window_count-1] = active_window = window;
    }*/

    for(n=0;n<window_count;n++)
      if(window_visibility_order[n] == window)
        break;
    for(;n<window_count-1;n++)
      window_visibility_order[n]=window_visibility_order[n+1];
    window_visibility_order[n]=active_window=window;
    
/*    if(titlebar_drag)
    {
      titlebar_drag->x0 = windows[window]->x;
      titlebar_drag->y0 = windows[window]->y;
      titlebar_drag->x1 = windows[window]->x+windows[window]->width;
      titlebar_drag->y1 = windows[window]->y+_FONT_HEIGHT+2;
    }*/
    update_titlebar(windows[window]);
    
    if(window_count>1 && !windows[window_visibility_order[n-1]]->minimized && draw)
      draw_window(windows[window_visibility_order[n-1]], 0, 2);
    if(window_visibility_order[n-1] != active_window && draw)
      draw_window(windows[active_window], 1, 0);
  }
  else
    active_window=window;
}
