//#define _FONT_HEIGHT 8

/*
  This function loads fonts
  Arguments: path to the font file as string,
  pointer to the pointer where the memory for the font data is allocated
*/

void load_system_font(char *font_file, unsigned char **font_array)
{
  FILE *fp;
  unsigned int font_file_len=0;
  if(!(fp = fopen(font_file, "rb")))
    return;

  while(fgetc(fp) != -1)
    font_file_len++;
  rewind(fp);

  if(!(*font_array = calloc(font_file_len, sizeof(char))))
  {
    nomemory();
    return;
  }

//  fgets(system_font, font_file_len, fp);
  fread(*font_array, sizeof(char), font_file_len, fp);

  fclose(fp);
}

/*
  This function draws a single character to screen
  Arguments: charnum = the character to be drawn, 
  x and y coordinates on the screen,
  color, font height and pointer to the font data
  
  Returns: width of the drawn char in pixels
*/  

unsigned char draw_char(unsigned char charnum, unsigned int charx, unsigned int chary, unsigned char color, unsigned char font_height, unsigned char *cp)
{
  unsigned char n,m;
  unsigned char c;
  unsigned char cw, cl;
  unsigned char retval=0;
  unsigned int x = charx;
  unsigned int y = chary;

  charnum-=33;
  
  for(n=0;n<222;n++,x=charx,y=chary)
  {
    // cw = width of the character (the first byte of every char)
    // can be 1, 2, 4 or 8 (pixels)
    cw = *cp++;
/*    if(cw>2)
      retval=cw-1;
    else
      retval=cw;*/
    // cl = length of the character (calculated from cw and font height)
    cl = (font_height * cw) >> 3;

    // read character data
    if(n==charnum)
    {
      for(m=cl;m;m--,y++,x=charx)
      {
        c = *cp++;
        switch(cw)
        {
          case 8:
            // first bit from the left
            if(c & 0x80)
              put_pixel(x, y, color);
            x++;
            // and so on
            if(c & 0x40)
              put_pixel(x, y, color);
            x++;
            if(c & 0x20)
              put_pixel(x, y, color);
            x++;
            if(c & 0x10)
              put_pixel(x, y, color);
            x++;
            if(c & 0x08)
              put_pixel(x, y, color);
            x++;
            if(c & 0x04)
              put_pixel(x, y, color);
            x++;
            if(c & 0x02)
            {
              put_pixel(x, y, color);
              if(!retval) retval++;
            }
            x++;
            if(c & 0x01)
            {
              put_pixel(x, y, color);
              retval=2;
            }
          break;
          case 4:
            // first bit from the left
            if(c & 0x80)
              put_pixel(x, y, color);
            x++;
            // and so on
            if(c & 0x40)
              put_pixel(x, y, color);
            x++;
            if(c & 0x20)
              put_pixel(x, y, color);
            x++;
            if(c & 0x10)
            {
              put_pixel(x, y, color);
              if(!retval) retval++;
            }
            x=charx; y++;
            // and the row increments              
            if(c & 0x08)
              put_pixel(x, y, color);
            x++;
            if(c & 0x04)
              put_pixel(x, y, color);
            x++;
            if(c & 0x02)
              put_pixel(x, y, color);
            x++;
            if(c & 0x01)
            {
              put_pixel(x, y, color);
              if(!retval) retval++;
            }
          break;
          case 2:
            if(c & 0x80)
              put_pixel(x, y, color);
            x++;
            if(c & 0x40)
              put_pixel(x, y, color);
            x--; y++;
            if(c & 0x20)
              put_pixel(x, y, color);
            x++;
            if(c & 0x10)
              put_pixel(x, y, color);
            x--; y++;
            if(c & 0x08)
              put_pixel(x, y, color);
            x++;
            if(c & 0x04)
              put_pixel(x, y, color);
            x--; y++;
            if(c & 0x02)
              put_pixel(x, y, color);
            x++;
            if(c & 0x01)
              put_pixel(x, y, color);
          break;
          case 1:
            if(c & 0x80)
              put_pixel(x, y, color);
            y++;
            if(c & 0x40)
              put_pixel(x, y, color);
            y++;
            if(c & 0x20)
              put_pixel(x, y, color);
            y++;
            if(c & 0x10)
              put_pixel(x, y, color);
            y++;
            if(c & 0x08)
              put_pixel(x, y, color);
            y++;
            if(c & 0x04)
              put_pixel(x, y, color);
            y++;
            if(c & 0x02)
              put_pixel(x, y, color);
            y++;
            if(c & 0x01)
              put_pixel(x, y, color);
          break;
        }
      }
      if(cw==8)
        return --cw+retval;
      else if (cw==4)
        return cw+retval;
      else if(cw==2)
        return 3;  
      else if(cw==1)
        return 2;
    }
    else
      cp+=cl;
  }
  return 0;
}
  
/*
  This function calls the above function in loop to print a string to the screen
  Arguments: x and y coordinates on the screen, 32-bit pointer to the string, 
  text color, font height in pixels, the X coordinate where to stop printing,
  and if monospace != 0 then prints the text with monospace font
*/  

void gprint_text(unsigned int x, unsigned int y, unsigned char __far *text, unsigned char color, unsigned char font_height, unsigned int x2, unsigned monospace)
{
  unsigned int xu;
  do
  {
    if(*text == 32)
    {
      if(!monospace)
         x+=4;
      else
        x += 9;
    }
    else if(*text == 9)
    {
      x+=30;
    }
    // underline next char
    else if(*text == 0x11)
    {
      xu = x+draw_char((*++text), x, y, color, font_height, system_font);
      draw_line(x, y+font_height-1, xu-2, y+font_height-1, color);
      x = xu;
    }
    else
    {
      if(!monospace)
        x += draw_char((*text), x, y, color, font_height, system_font);
      else
      {
        draw_char((*text), x, y, color, font_height, monospace_font);
        x+=9;
      }
    }
  }
  while(*text++ && x+8<x2);
}

struct text_print cprint_text(unsigned int x, unsigned int y, unsigned char *text, unsigned char color, unsigned char font_height, unsigned int x2, unsigned int count)
{
  struct text_print retval;
  unsigned int n=0;
  if(!count) count--;

  if(*text)
    do
    {
      if(*text == 32) x+=4; 
      else
        x += draw_char((*text), x, y, color, font_height, system_font);
    }
    while(*++text && x+8<x2 && ++n < count);

  retval.text = text;
  retval.x = x;

  return retval;
}
