char *mouse_cursor = 0;

/*
Loads the mouse cursor bitmap
*/
void load_mouse_cursor(char *filename)
{
  FILE *fp;
  mouse_cursor = malloc(12*sizeof(char));

  fp = fopen(filename, "rb");
  fread(mouse_cursor, sizeof(char), 12, fp);
  fclose(fp);  
}

/*
Detects the mouse
Returs: count of buttons if mouse detected, else returns 0
*/
int detect_mouse(void)
{
  int mouse;
  int buttons;
  __asm
  {
//    push ax
//    push bx
    mov ax, 0x00
    int 0x33
    mov mouse, ax
    mov buttons, bx
//    pop bx
//    pop ax
  }
  if(mouse)
  {
    titlebar_drag = malloc(sizeof(struct screen_region));
    return buttons;
  }
  return 0;
}

/* Initializes the mouse and returns a pointer to the mouse status struct
Returns 0 if no mouse detected */
struct mouse_status *init_mouse(void)
{
  int buttons;
  struct mouse_status *retval = 0;

  if((buttons = detect_mouse()))
  {
    retval = calloc(1, sizeof(struct mouse_status));
    retval->pointer_x=160;
    retval->pointer_y=100;
    load_mouse_cursor("normal.cur");
  }

  return retval;
}
/*
Detects mouse movement
Needs a pointer to the mouse status struct as an argument
Returns: 0 if no mouse movement
Else If mouse movement detected, returns a pointer to a signed int array
where the offset 0 is the movement in x axis and offset 1 is the movement in
y axis
*/
int *get_mouse_movement(void)
{
  static int retval[2];
  int d_x=0;
  int d_y=0;
  unsigned char bl_reg;
 
  __asm
  {
    mov ax, 0x0B
    int 0x33
    mov d_x, cx
    mov d_y, dx
    mov ax, 0x03
    int 0x33
    mov bl_reg, bl
  }

  mouse->pointer_x+=d_x;
  mouse->pointer_y+=d_y;

  if(mouse->pointer_x < 0)
  {
    d_x -= mouse->pointer_x;
    mouse->pointer_x = 0;
  }
  if(mouse->pointer_y < 0)
  {
    d_y -= mouse->pointer_y;    
    mouse->pointer_y = 0;
  }
  if(mouse->pointer_x >= _RES_X)
  {
    d_x -= mouse->pointer_x-_RES_X;    
    mouse->pointer_x = _RES_X-1;
  }
  if(mouse->pointer_y >= _RES_Y)
  {
    d_y -= mouse->pointer_y-_RES_Y;    
    mouse->pointer_y = _RES_Y-1;
  }

  if((bl_reg & 0x01))
    mouse->left_pressed=1;
  else
    mouse->left_pressed=0;
  if((bl_reg & 0x02))
    mouse->right_pressed=1;
  else
    mouse->right_pressed=0;

  if(d_x || d_y)
  {
    retval[0] = d_x;
    retval[1] = d_y;
    return retval;
  }
/*  if(d_x || d_y)
    return 1;*/
/*  if(d_x > 0) return 1;
  if(d_y > 0) return 2;
  if(d_x < 0) return 3;
  if(d_y < 0) return 4;*/
  return 0;
}

/*
Draws mouse cursor
Argument 1: pointer to the mouse status struct, 2: pointer to the video memory or buffer,
3: unsigned int updatebg

if updatebg == 0, draws mouse cursor normally
if updatebg == 2, hides mouse cursor
if updatebg == 1, re-draws mouse cursor
*/
void draw_mouse_cursor(unsigned updatebg)
{
  static unsigned char background[12][8];
  static unsigned int old_x = 160;
  static unsigned int old_y = 100;
  unsigned int n, m;

  if(!mouse) return;

  if(!updatebg || updatebg == 2)
    for(n=12;n--;)
      for(m=8;m--;)
        put_pixel(old_x+m, old_y+n, background[n][m]);
  if(!updatebg || updatebg == 1)
    for(n=12;n--;)
      for(m=8;m--;)      
        background[n][m] = get_pixel(mouse->pointer_x+m, mouse->pointer_y+n);

  if(!updatebg)
  {  
    old_x = mouse->pointer_x;
    old_y = mouse->pointer_y;

    for(n=12;n--;)
      for(m=8;m--;)
        if((mouse_cursor[n]<<m)&0x80)
          put_pixel(mouse->pointer_x+m, mouse->pointer_y+n, _CURSOR_COLOR);
  }
  
/*  put_pixel(old_x, old_y, background, dest);
  background = get_pixel(mouse->pointer_x, mouse->pointer_y, dest);
  old_x = mouse->pointer_x;
  old_y = mouse->pointer_y;
  put_pixel(mouse->pointer_x, mouse->pointer_y, _CURSOR_COLOR, dest);*/
}

void wait_leftbutton_release(void)
{
  while(mouse->left_pressed)
  {
    if(get_mouse_movement())
      draw_mouse_cursor(0);
  }
}

/*
Starts drawing the border when moving or resizing a window
*/
void dwwindow_initborder(struct prog_window *window)
{
  while(mouse->left_pressed)
  {
    if(get_mouse_movement())
    {  
      window->hidden=1;      
      draw_screen();
      draw_move_borders(window->x, window->y, window->width, window->height+2+_FONT_HEIGHT, 0);
      break;
    }
  }
}
/*
Ends drawing the border and frees all mallocs
*/
void dwwindow_endborder(struct prog_window *window)
{
  draw_move_borders(window->x, window->y, window->width, window->height+2+_FONT_HEIGHT, 2);
  window->hidden=0; 
}

/*
Moves the window by mouse
*/
void drag_window(struct prog_window *window)
{
  int *d=0;

  dwwindow_initborder(window);
  while(mouse->left_pressed)
  {
    if((d=get_mouse_movement()))
    {
      window->x += d[0];
      window->y += d[1];
      titlebar_drag->x0 +=d[0];
      titlebar_drag->y0 +=d[1];
      titlebar_drag->x1 +=d[0];
      titlebar_drag->y1 +=d[1];
      draw_move_borders(window->x, window->y, window->width, window->height+2+_FONT_HEIGHT, 1);
    }
  }
  dwwindow_endborder(window); 
}

/*
Resizes the window by mouse
*/
void mouse_resize_window(struct prog_window *window)
{
  int *d=0;
  dwwindow_initborder(window);
  while(mouse->left_pressed)
  {
    if((d=get_mouse_movement()))
    {
      if((window->width += d[0]) < 120) window->width=120;
      if((window->height += d[1]) < 120) window->height=120;
      if((titlebar_drag->x1 += d[0]) < 120) titlebar_drag->x1 = 120;
      draw_move_borders(window->x, window->y, window->width, window->height+2+_FONT_HEIGHT, 3);      
    }
  }
  dwwindow_endborder(window);  
}

/*
Moves a desktop icon or shortcut by mouse
*/
void mouse_drag_icon(void *icon, unsigned type)
{
  int *d;
  while(mouse->left_pressed)
  {
    if((d=get_mouse_movement()))
    {
      if(!type)
      {
        ((struct icon *)icon)->x +=d[0];
        ((struct icon *)icon)->y +=d[1];
      }
      else
      {
        ((struct desktop_shortcut *)icon)->x +=d[0];
        ((struct desktop_shortcut *)icon)->y +=d[1];
      }
      draw_desktop();
    }
  }
}

/*void mouse_drag_shortcut(struct desktop_shortcut *icon)
{
  int *d;
  while(mouse->left_pressed)
  {
    if((d=get_mouse_movement()))
    {
      icon->x +=d[0];
      icon->y +=d[1];
      draw_desktop();
    }
  }
  draw_screen();
}*/

/*
Handler for the left mouse button (desktop)
*/
int mleft_handler_desktop(void)
{
  unsigned int n = desktop_icon_count;
  unsigned int m = 0;
  unsigned int l = 0;
  int *d = 0;
  unsigned int icon_selected_prev = icon_selected;
//  while(!(d=get_mouse_movement()) && mouse->left_pressed);

  draw_mouse_cursor(2);  
  // if mouse pointer is over a desktop icon
  while(n--)
  {
    if(mouse->pointer_x > desktop_icons[n].x && mouse->pointer_x < desktop_icons[n].x+40
      && mouse->pointer_y > desktop_icons[n].y && mouse->pointer_y < desktop_icons[n].y+35)
    {
      while(!(d=get_mouse_movement()) && mouse->left_pressed);
      if(!d)
      {
        if(icon_selected_prev != n)
          icon_selected = n;
        else
          return -6;
        return -7;
      }
      // break while if mouse movement detected
      else break;
    }
  }

  if(!d)
  {
    // if mouse pointer is over a shortcut icon
    for(m=shortcut_count;m--;)
    {
      if(mouse->pointer_x > desktop_shortcuts[m].x && mouse->pointer_x < desktop_shortcuts[m].x+40
      && mouse->pointer_y > desktop_shortcuts[m].y && mouse->pointer_y < desktop_shortcuts[m].y+35)
      {
        while(!(d=get_mouse_movement()) && mouse->left_pressed);
        if(!d)
        {
          if(icon_selected_prev != desktop_icon_count+m)
            icon_selected = desktop_icon_count+m;
          else
            return -6;
          return -7;
        }
        else break;
      }
    }
    // if mouse pointer is over a minimized window icon (if d!=0 then jump to else)
    if(!d)
    {
      for(m=0;l<window_count;l++)
      {
        if(windows[l]->minimized)
        {
          if(mouse->pointer_x >= _WINDOW_ICONS_LEFT+m*_WINDOW_ICONS_MARGIN && mouse->pointer_x < _WINDOW_ICONS_LEFT+m*_WINDOW_ICONS_MARGIN+_WINDOW_ICONS_MARGIN-2
            && mouse->pointer_y > _RES_Y-40)
          {
            // draw inset borders while mouse leftbutton down
            while(!(d=get_mouse_movement()) && mouse->left_pressed)
            {
              draw_borders(_WINDOW_ICONS_LEFT+m*_WINDOW_ICONS_MARGIN, _RES_Y-40,
                _WINDOW_ICONS_MARGIN-2, 39, 1);
              draw_mouse_cursor(0);
            }
            if(!d)
            {
              unminimize_window(l);
              draw_mouse_cursor(1);            
              return 0;
            }
            else
            {
              draw_borders(_WINDOW_ICONS_LEFT+m*_WINDOW_ICONS_MARGIN, _RES_Y-40,
                _WINDOW_ICONS_MARGIN-2, 39, 0);
              return 0;
            }
          }
          m++;
        }
      }
    }
    // mouse motion detected while left button down && mouse over a shortcut
    else
    {
    //  mouse_drag_shortcut(&desktop_shortcuts[m]);
      mouse_drag_icon((void*)&desktop_shortcuts[m], 1);
      save_desktop_shortcut(m);      
      return -9;
    }
  }
  // if d != 0 (mouse motion detected while left button down && mouse over a desktop icon)
  else
  {
//    draw_mouse_cursor(2);    
    mouse_drag_icon((void*)&desktop_icons[n], 0);
//    draw_mouse_cursor(1);    
    return -9;
  }
  return 0;
}

/*
Handler for scrollbars
*/
void mouse_scrollbar_handler(struct prog_window *window, struct hitbox *hitbox)
{
  int *d;
  // vertical scrollbar
  if(!window->window_grid[hitbox->element]->ctype)
  {
    // upper arrow button pressed
    if(mouse->pointer_y < window->y+hitbox->y0+12)
    {
      draw_borders(window->x+hitbox->x0, window->y+hitbox->y0,
          12, 12, 1);
        // scroll up...
      if(window->window_grid[hitbox->element]->scroll_x) window->window_grid[hitbox->element]->scroll_x--;
      wait_leftbutton_release();
/*      while(mouse->left_pressed)
      {
        if(get_mouse_movement())
          draw_mouse_cursor(0);
      }      */
      draw_borders(window->x+hitbox->x0, window->y+hitbox->y0,
          12, 12, 0);      
    }
      // if mouse pointer is between the upper arrow button and the anchor
    else if(window->window_grid[hitbox->element]->c && mouse->pointer_y < window->y+hitbox->y0+13
    + (unsigned long)window->window_grid[hitbox->element]->scroll_x * (window->window_grid[hitbox->element]->width-12) / window->window_grid[hitbox->element]->c)
    {
      wait_leftbutton_release();      
/*      while(mouse->left_pressed) 
      {
        if(get_mouse_movement())
          draw_mouse_cursor(0);
      }*/
      // scroll up amount of scrollbar->width...
      if(window->window_grid[hitbox->element]->scroll_x >= window->window_grid[hitbox->element]->width/(_FONT_HEIGHT+2))
        window->window_grid[hitbox->element]->scroll_x -= window->window_grid[hitbox->element]->width/(_FONT_HEIGHT+2);
      else
        window->window_grid[hitbox->element]->scroll_x = 0;
    }
        // if mouse pointer is over the anchor        
    else if(window->window_grid[hitbox->element]->c && mouse->pointer_y < window->y+hitbox->y0+25
     + (unsigned long)window->window_grid[hitbox->element]->scroll_x * (window->window_grid[hitbox->element]->width-12) / window->window_grid[hitbox->element]->c)
    {
      while(mouse->left_pressed) 
      {
        if((d = get_mouse_movement()))
        {
          draw_mouse_cursor(0);
          // scroll by d[1]...
          if((int)window->window_grid[hitbox->element]->scroll_x+d[1] >= 0 && window->window_grid[hitbox->element]->scroll_x+d[1] <= window->window_grid[hitbox->element]->c)
              window->window_grid[hitbox->element]->scroll_x+=d[1];
          window_drawelement(window, hitbox->element);
          if(window->window_grid[hitbox->element]->connect_value)
            *window->window_grid[hitbox->element]->connect_value = window->window_grid[hitbox->element]->scroll_x;
          if(window->window_grid[hitbox->element]->connect_element != -1)
            window_drawelement(window, window->window_grid[hitbox->element]->connect_element);          
        }
      }
    }
    // if mouse pointer is between the anchor and the down arrow button
    else if(mouse->pointer_y < window->y+hitbox->y1-12)
    {
      wait_leftbutton_release();      
/*      while(mouse->left_pressed) 
      {
        if(get_mouse_movement())
          draw_mouse_cursor(0);
      }*/
      // scroll down amount of scrollbar->width...
      if(window->window_grid[hitbox->element]->scroll_x+window->window_grid[hitbox->element]->width/(_FONT_HEIGHT+2) < window->window_grid[hitbox->element]->c)
        window->window_grid[hitbox->element]->scroll_x += window->window_grid[hitbox->element]->width/(_FONT_HEIGHT+2);
      else
        window->window_grid[hitbox->element]->scroll_x = window->window_grid[hitbox->element]->c;        
    }
    // down arrow button pressed
    else
    {
      draw_borders(window->x+hitbox->x0, window->y+hitbox->y1-12,
          12, 12, 1);
        // scroll down...
      if(window->window_grid[hitbox->element]->c && window->window_grid[hitbox->element]->scroll_x < window->window_grid[hitbox->element]->c)
        window->window_grid[hitbox->element]->scroll_x++;
      wait_leftbutton_release();        
/*      while(mouse->left_pressed)
      {
        if(get_mouse_movement())
          draw_mouse_cursor(0);
      }      */
      draw_borders(window->x+hitbox->x0, window->y+hitbox->y1-12,
          12, 12, 0);     
    }
    window_drawelement(window, hitbox->element);
    if(window->window_grid[hitbox->element]->connect_value)
      *window->window_grid[hitbox->element]->connect_value = window->window_grid[hitbox->element]->scroll_x;
    if(window->window_grid[hitbox->element]->connect_element != -1)
      window_drawelement(window, window->window_grid[hitbox->element]->connect_element);                        
  }
  // horizontal scrollbar
  else
  {
    // left arrow button pressed
    if(mouse->pointer_x < window->x+hitbox->x0+12)
    {
      draw_borders(window->x+hitbox->x0, window->y+hitbox->y0,
          12, 12, 1);
        // scroll left...
      if(window->window_grid[hitbox->element]->scroll_x) window->window_grid[hitbox->element]->scroll_x--;
      wait_leftbutton_release();
/*      while(mouse->left_pressed)
      {
        if(get_mouse_movement())
          draw_mouse_cursor(0);
      }      */
      draw_borders(window->x+hitbox->x0, window->y+hitbox->y0,
          12, 12, 0);      
    }
      // if mouse pointer is between the left arrow button and the anchor
    else if(window->window_grid[hitbox->element]->c && mouse->pointer_x < window->x+hitbox->x0+13
    + (unsigned long)window->window_grid[hitbox->element]->scroll_x * (window->window_grid[hitbox->element]->width-12) / window->window_grid[hitbox->element]->c)
    {
      wait_leftbutton_release();      
/*      while(mouse->left_pressed) 
      {
        if(get_mouse_movement())
          draw_mouse_cursor(0);
      }*/
      // scroll left amount of scrollbar->width...
      if(window->window_grid[hitbox->element]->scroll_x >= window->window_grid[hitbox->element]->width/(_FONT_HEIGHT+2))
        window->window_grid[hitbox->element]->scroll_x -= window->window_grid[hitbox->element]->width/(_FONT_HEIGHT+2);
      else
        window->window_grid[hitbox->element]->scroll_x = 0;
    }
        // if mouse pointer is over the anchor        
    else if(window->window_grid[hitbox->element]->c && mouse->pointer_x < window->x+hitbox->x0+25
     + (unsigned long)window->window_grid[hitbox->element]->scroll_x * (window->window_grid[hitbox->element]->width-12) / window->window_grid[hitbox->element]->c)
    {
      while(mouse->left_pressed) 
      {
        if((d = get_mouse_movement()))
        {
          draw_mouse_cursor(0);
          // scroll by d[0]...
          if((int)window->window_grid[hitbox->element]->scroll_x+d[0] >= 0 && window->window_grid[hitbox->element]->scroll_x+d[0] <= window->window_grid[hitbox->element]->c)
              window->window_grid[hitbox->element]->scroll_x+=d[0];
          window_drawelement(window, hitbox->element);
          if(window->window_grid[hitbox->element]->connect_value)
            *window->window_grid[hitbox->element]->connect_value = window->window_grid[hitbox->element]->scroll_x;
          if(window->window_grid[hitbox->element]->connect_element != -1)
            window_drawelement(window, window->window_grid[hitbox->element]->connect_element);                    
        }
      }
    }
    // if mouse pointer is between the anchor and the down arrow button
    else if(mouse->pointer_x < window->x+hitbox->x1-12)
    {
      wait_leftbutton_release();      
/*      while(mouse->left_pressed) 
      {
        if(get_mouse_movement())
          draw_mouse_cursor(0);
      }*/
      // scroll right by amount of scrollbar->width...
      if(window->window_grid[hitbox->element]->scroll_x+window->window_grid[hitbox->element]->width/(_FONT_HEIGHT+2) < window->window_grid[hitbox->element]->c)
        window->window_grid[hitbox->element]->scroll_x += window->window_grid[hitbox->element]->width/(_FONT_HEIGHT+2);
      else
        window->window_grid[hitbox->element]->scroll_x = window->window_grid[hitbox->element]->c;        
    }
    // right arrow button pressed
    else
    {
      draw_borders(window->x+hitbox->x1-12, window->y+hitbox->y0,
          12, 12, 1);
        // scroll right...
      if(window->window_grid[hitbox->element]->c && window->window_grid[hitbox->element]->scroll_x < window->window_grid[hitbox->element]->c)
        window->window_grid[hitbox->element]->scroll_x++;
      wait_leftbutton_release();        
/*      while(mouse->left_pressed)
      {
        if(get_mouse_movement())
          draw_mouse_cursor(0);
      }      */
      draw_borders(window->x+hitbox->x1-12, window->y+hitbox->y0,
          12, 12, 0);     
    }
    window_drawelement(window, hitbox->element);
    if(window->window_grid[hitbox->element]->connect_value)
      *window->window_grid[hitbox->element]->connect_value = window->window_grid[hitbox->element]->scroll_x;
    if(window->window_grid[hitbox->element]->connect_element != -1)
      window_drawelement(window, window->window_grid[hitbox->element]->connect_element);              
  }
}

/*
Handler for window_list-elements
*/
void mouse_list_handler(struct prog_window *window, struct hitbox *hitbox)
{
  draw_mouse_cursor(2);
  if(window->window_grid[hitbox->element]->active)
  {
    if(window->window_grid[hitbox->element]->selectc != window->window_grid[hitbox->element]->scroll_y
      + (mouse->pointer_y-(window->y+2+_FONT_HEIGHT+window->window_grid[hitbox->element]->y)) / (2+_FONT_HEIGHT))
    {
      window->window_grid[hitbox->element]->selectc = window->window_grid[hitbox->element]->scroll_y
      + (mouse->pointer_y-(window->y+2+_FONT_HEIGHT+window->window_grid[hitbox->element]->y)) / (2+_FONT_HEIGHT);
      window_drawelement(window, hitbox->element);      
  
      while(mouse->left_pressed) 
        get_mouse_movement();
    }
    else
    {
      window->window_grid[hitbox->element]->connect_element = window->window_grid[hitbox->element]->scroll_y
      + (mouse->pointer_y-(window->y+2+_FONT_HEIGHT+window->window_grid[hitbox->element]->y)) / (2+_FONT_HEIGHT);
      while(mouse->left_pressed) 
        get_mouse_movement();    
    }
  }
  draw_mouse_cursor(1);    
}

/*
Handler for left mouse button
*/
int mleft_handler(struct prog_window *window)
{
  unsigned int n;
  draw_mouse_cursor(2);      
  if(window)
  {
    if(window->resizable)
    {
      // if mouse pointer is over the windows's rightdown corner
      if(mouse->pointer_x >= window->x+window->width-1 && mouse->pointer_x <= window->x+window->width+1
      && mouse->pointer_y >= window->y+window->height+_FONT_HEIGHT+1 && mouse->pointer_y <= window->y+window->height+_FONT_HEIGHT+3)
        return -8;
    // if mouse pointer is over the maximize/unmaximize button                
      if(mouse->pointer_x > window->x+window->width-_FONT_HEIGHT-2 && mouse->pointer_x < window->x+window->width-1
        && mouse->pointer_y > window->y && mouse->pointer_y < window->y+_FONT_HEIGHT+1)
        return -4;
    }
    // if mouse pointer is over the minimize/unminimize button
    if(window->minimizable)
      if(mouse->pointer_x >= window->x+window->width-2*(_FONT_HEIGHT+1) && mouse->pointer_x < window->x+window->width-_FONT_HEIGHT-2
        && mouse->pointer_y > window->y && mouse->pointer_y < window->y+_FONT_HEIGHT+1)
        return -5;
    // if mouse pointer is over the top left dropdown menu button
    if(mouse->pointer_x > window->x && mouse->pointer_x < window->x+_FONT_HEIGHT+1
      && mouse->pointer_y > window->y && mouse->pointer_y < window->y+_FONT_HEIGHT+1)
      return -2;        
    // if mouse pointer is over the titlebar
    if(!window->maximized)
      if(mouse->pointer_x >= titlebar_drag->x0 && mouse->pointer_y >= titlebar_drag->y0
        && mouse->pointer_x < titlebar_drag->x1 && mouse->pointer_y < titlebar_drag->y1)
        return -1;
    // if mouse pointer is over a hitbox
    for(n=window->hitbox_count;n--;)
    {
      if(mouse->pointer_x >= window->x + window->hitbox[n].x0 && mouse->pointer_x < window->x + window->hitbox[n].x1
      && mouse->pointer_y >= window->y + window->hitbox[n].y0 && mouse->pointer_y < window->y + window->hitbox[n].y1)
      {
        switch(window->window_grid[window->hitbox[n].element]->type)
        {
          // if list
          case 3:
            mouse_list_handler(window, &window->hitbox[n]);
          break;
          // if scrollbar
          case 4:
            mouse_scrollbar_handler(window, &window->hitbox[n]);
            draw_mouse_cursor(2);
            window_drawelement(window, window->hitbox[n].element);
            draw_mouse_cursor(1);            
          break;
          // button, switch to inset border
          case 1: case 5:
            while(mouse->left_pressed)
            {
              if(get_mouse_movement())
                draw_mouse_cursor(0);
              draw_borders(window->x+window->hitbox[n].x0, window->y+window->hitbox[n].y0,
                  window->hitbox[n].x1-window->hitbox[n].x0, window->hitbox[n].y1-window->hitbox[n].y0, 1);                        
            }
            draw_borders(window->x+window->hitbox[n].x0, window->y+window->hitbox[n].y0,
              window->hitbox[n].x1-window->hitbox[n].x0, window->hitbox[n].y1-window->hitbox[n].y0, 0);
          break;
        }
        if(mouse->pointer_x >= window->x + window->hitbox[n].x0 && mouse->pointer_x < window->x + window->hitbox[n].x1
        && mouse->pointer_y >= window->y + window->hitbox[n].y0 && mouse->pointer_y < window->y + window->hitbox[n].y1)
        {
          return n+100;
        }
      }
    }
    // if mouse pointer is outside the window
    if(window->type && !(mouse->pointer_x > window->x && mouse->pointer_x < window->x+window->width
      && mouse->pointer_y > window->y && mouse->pointer_y < window->y+window->height+_FONT_HEIGHT+2))
    {
      // if mouse pointer is over another window
      for(n=window_count;n--;)
      {
//        if(mouse->pointer_x > windows[n]->x && mouse->pointer_x < windows[n]->x+windows[n]->width
//          && mouse->pointer_y > windows[n]->y && mouse->pointer_y < windows[n]->y+windows[n]->height)
        if(mouse->pointer_x > windows[window_visibility_order[n]]->x && mouse->pointer_x < windows[window_visibility_order[n]]->x+windows[window_visibility_order[n]]->width
          && mouse->pointer_y > windows[window_visibility_order[n]]->y && mouse->pointer_y < windows[window_visibility_order[n]]->y+windows[window_visibility_order[n]]->height+_FONT_HEIGHT+2)
        {
          if(!windows[window_visibility_order[n]]->minimized)
          {
            draw_mouse_cursor(2);            
            set_active_window(window_visibility_order[n], 1);
            draw_mouse_cursor(1);            
            return -3;
          }
        }
      }
//      if(mleft_handler_desktop() == -6)
//        return -6;
      return mleft_handler_desktop();
    }
  }
  else
    return mleft_handler_desktop();  
//    if(mleft_handler_desktop() == -6)
//      return -6;
  return 0;
}

/*
Handler for the mouse...
*/
int mouse_handler(struct prog_window *window)
{
  int a;
  if(get_mouse_movement())
  {
    draw_mouse_cursor(0);
    screensaver_timer = 0;
  }
  // if a window is active and the left button is pressed      
  if(mouse->left_pressed)
  {
    screensaver_timer = 0;
    if(!(a = mleft_handler(window))) return 0;
    // a==1 -> mouse over titlebar
    else if(a == -1)
    {
      drag_window(window);
//      draw_window(window, 1, 0);
//      draw_mouse_cursor(1);      
    }
    else if(a == -2)
    {
//      if((a = 100 + window_menu(window, 0)) == 98) a = -1;
      // if topleft dropdown menu is used, return selection id (indexing starts from 1)
      if((a = window_menu(window, 0)) == -2) a = -1;
//      else a++;
    }
    else if(a == -8)
    {
      draw_mouse_cursor(1);      
      mouse_resize_window(window);
      window->resized=1;
      a = -1;
    }
    else if(a == -4 || a == -5)
    {
      if(a == -4)
        draw_borders(window->x+window->width-(_FONT_HEIGHT), window->y+2, _FONT_HEIGHT-2, _FONT_HEIGHT-2, 1);
      else
        draw_borders(window->x+window->width-2*(_FONT_HEIGHT+1)+1, window->y+2, _FONT_HEIGHT-2, _FONT_HEIGHT-2, 1);      
      while(mouse->left_pressed)
      {
        if(get_mouse_movement())
          draw_mouse_cursor(0);
        if(a != mleft_handler(window))
        {
          draw_window(window, 1, 2);
          return 0;
        }
      }
    }
    return a;
  } 
  return 0;
}
/*
Mouse handler for drop-down menus

Returns: if a menu item is selected, returns the selection id
Else returns -1 if nothing has happened, or -2 if the user clicked somewhere
outside the menu
*/
int menu_mouse_handler(struct prog_window *window, int x, int y, unsigned int width, unsigned int itemcount, unsigned int menuid)
{
  unsigned int selected;
      
  if(get_mouse_movement())
    draw_mouse_cursor(0);

  if(mouse->left_pressed)
  {
    if(mouse->pointer_x > x && mouse->pointer_x < x+width
      && mouse->pointer_y > y && mouse->pointer_y < y+ (_FONT_HEIGHT+2) * itemcount)
    {
      while(mouse->left_pressed)
      {
        if(get_mouse_movement())
          draw_mouse_cursor(0);
        if(selected != (mouse->pointer_y - y) / (_FONT_HEIGHT+2))
        {
          if((selected = (mouse->pointer_y - y) / (_FONT_HEIGHT+2)) < itemcount)
            print_menu(window, menuid, selected);
        }
      }
      if(mouse->pointer_x > x && mouse->pointer_x < x+width
        && mouse->pointer_y > y && mouse->pointer_y < y+ (_FONT_HEIGHT+2) * itemcount)
      {
        return 1+(mouse->pointer_y - y) / (_FONT_HEIGHT+2);
      }
    }
    return -2;
  }
  return -1;
}
